'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.GenericDatasource = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var GenericDatasource = exports.GenericDatasource = function () {
  function GenericDatasource(instanceSettings, $q, backendSrv, templateSrv) {
    _classCallCheck(this, GenericDatasource);

    this.type = instanceSettings.type;
    this.url = instanceSettings.url;
    this.name = instanceSettings.name;
    this.q = $q;
    this.backendSrv = backendSrv;
    this.templateSrv = templateSrv;

    this.spotinstToken = '';
    this.accountIdQueryParam = '';

    this.buildSpotinstAuthHeaders(instanceSettings);
  }

  _createClass(GenericDatasource, [{
    key: 'buildSpotinstAuthHeaders',
    value: function buildSpotinstAuthHeaders(instanceSettings) {

      if (instanceSettings.jsonData && instanceSettings.jsonData.accountId && instanceSettings.jsonData.spotinstToken) {

        this.accountIdQueryParam = "?accountId=" + instanceSettings.jsonData.accountId;
        this.spotinstToken = instanceSettings.jsonData.spotinstToken;
      }

      var jwtToken = 'Bearer ' + this.spotinstToken;

      this.headers = {
        'Content-Type': 'application/json',
        'Authorization': jwtToken
      };
    }
  }, {
    key: 'createPayloadWithAlias',
    value: function createPayloadWithAlias(queryTarget, options, dimensions, alias) {
      var origAlias = queryTarget.dimensionAlias;

      queryTarget.dimensionAlias = !_lodash2.default.isEmpty(origAlias) ? origAlias + '-' + alias : alias;

      var payload = this.createPayload(queryTarget, options, dimensions);
      queryTarget.dimensionAlias = origAlias;

      return payload;
    }
  }, {
    key: 'createPayload',
    value: function createPayload(queryTarget, options, dimensions) {
      var qNamespace = void 0;
      var qMetricName = void 0;
      var qStat = void 0;
      var qInterval = void 0;

      if (queryTarget.namespace !== 'select namespace') {
        qNamespace = queryTarget.namespace;
      }

      if (queryTarget.metricName !== 'select metric') {
        qMetricName = queryTarget.metricName;
      }

      if (queryTarget.interval) {
        qInterval = queryTarget.interval;
      }

      if (queryTarget.statistic) {
        qStat = queryTarget.statistic;
      }

      if (qNamespace && qMetricName && qStat && qInterval) {

        var startTime = options.range.from;
        var endTime = options.range.to;
        var qDimAlias = queryTarget.dimensionAlias;
        var targetValue = qDimAlias !== null && qDimAlias !== '' ? qDimAlias : '';

        return {
          refId: queryTarget.refId,
          hide: queryTarget.hide,
          target: targetValue,
          payload: {
            namespace: qNamespace,
            metricName: qMetricName,
            statistic: qStat,
            timeInterval: qInterval,
            dateRange: {
              from: startTime,
              to: endTime
            },
            dimensions: dimensions
          }
        };
      } else {
        return null;
      }
    }
  }, {
    key: 'buildQueryParameters',
    value: function buildQueryParameters(options) {
      var _this = this;

      if (options && options.range && options.targets && options.targets.length > 0) {

        var payloadArr = [];

        options.targets.forEach(function (queryTarget) {

          // Filter out placeholders
          var validDimensionsArr = queryTarget.dimensionArray.filter(function (dimension) {
            return dimension.key !== 'select namespace' && dimension.value !== 'dimension value';
          });

          // Check if there's usage for template variable
          var templateVarUsage = validDimensionsArr.filter(function (dim) {
            return _this.templateSrv.variableExists(dim.value);
          });

          if (_lodash2.default.isEmpty(templateVarUsage)) {
            var dimensionToQuery = validDimensionsArr.map(function (dimension) {
              return {
                name: dimension.key,
                value: dimension.value
              };
            });

            var payload = _this.createPayload(queryTarget, options, dimensionToQuery);
            if (payload !== null) {
              payloadArr.push(payload);
            }
          } else {
            // Set aside the query variables which aren't part of the dynamic template
            var staticDimensions = [];

            // Currently we're supporting only single template variable usage per query
            var templateValuesKey = null;
            var templateValuesDimensions = null;

            // Map template variable and static variables
            validDimensionsArr.forEach(function (dim) {

              if (_this.templateSrv.variableExists(dim.value)) {
                var templateValuesStr = _this.templateSrv.replace(dim.value).replace(/[{}]/g, '');
                templateValuesDimensions = templateValuesStr.split(',');
                templateValuesKey = dim.key;
              } else {
                var staticDimension = {
                  name: dim.key,
                  value: dim.value
                };
                staticDimensions.push(staticDimension);
              }
            });

            if (templateValuesDimensions !== null) {

              templateValuesDimensions.forEach(function (tVal) {

                var templateVarDimension = {
                  name: templateValuesKey,
                  value: tVal
                };

                // Create a new array address but keep the content by reference
                var dimensions = staticDimensions.slice(0);
                dimensions.push(templateVarDimension);

                var payload = _this.createPayloadWithAlias(queryTarget, options, dimensions, tVal);

                payloadArr.push(payload);
              });
            }
          }
        });

        options.targets = payloadArr;
      }
      return options;
    }
  }, {
    key: 'query',
    value: function query(options) {
      var query = this.buildQueryParameters(options);
      query.targets = query.targets.filter(function (t) {
        return t && !t.hide;
      });

      if (query.targets.length <= 0) {
        return this.q.when({ data: [] });
      }

      if (query.range) {
        return this.backendSrv.datasourceRequest({
          url: this.buildSpotinstUrl('/query'),
          data: query,
          method: 'POST',
          headers: this.headers
        });
      } else {
        // return this.backendSrv.datasourceRequest({
        //     url: this.url + '/query',
        //     data: query,
        //     method: 'POST',
        //     headers: this.headers
        // });

        // TODO: Need to fix, generate errors: Cannot read property 'then' of
        // undefined
        return this.q.when({ data: [] });
      }
    }
  }, {
    key: 'testDatasource',
    value: function testDatasource() {
      return this.backendSrv.datasourceRequest({
        url: this.buildSpotinstUrl('/'),
        method: 'GET',
        headers: this.headers
      }).then(function (response) {
        if (response.status === 200) {
          return {
            status: "success",
            message: "Data source is working",
            title: "Success"
          };
        }
      });
    }
  }, {
    key: 'annotationQuery',
    value: function annotationQuery(options) {
      var query = this.templateSrv.replace(options.annotation.query, {}, 'glob');
      var annotationQuery = {
        range: options.range,
        annotation: {
          name: options.annotation.name,
          datasource: options.annotation.datasource,
          enable: options.annotation.enable,
          iconColor: options.annotation.iconColor,
          query: query
        },
        rangeRaw: options.rangeRaw
      };

      return this.backendSrv.datasourceRequest({
        url: this.buildSpotinstUrl('/annotations'),
        method: 'POST',
        headers: this.headers,
        data: annotationQuery
      }).then(function (result) {
        return result.data;
      });
    }
  }, {
    key: 'metricFindQuery',
    value: function metricFindQuery(options) {
      var target = void 0;
      var isTemplate = false;

      if (typeof options === "string") {
        try {
          var json = JSON.parse(options);
          isTemplate = true;
          target = json;
        } catch (e) {
          target = options;
        }
      } else {
        target = options.target;
      }

      var interpolated = void 0;
      if (isTemplate) {
        interpolated = {
          namespace: target.namespace,
          target: target.dimension
        };
      } else {
        interpolated = {
          target: this.templateSrv.replace(target, null, 'regex')
        };
      }

      return this.backendSrv.datasourceRequest({
        url: this.buildSpotinstUrl('/search'),
        data: interpolated,
        method: 'POST',
        headers: this.headers
      }).then(this.mapToTextValue);
    }
  }, {
    key: 'getNamespaces',
    value: function getNamespaces(options) {
      return this.backendSrv.datasourceRequest({
        url: this.buildSpotinstUrl('/namespaces'),
        data: null,
        method: 'GET',
        headers: this.headers
      }).then(this.mapToTextValue);
    }
  }, {
    key: 'getMetrics',
    value: function getMetrics(options) {
      var qParams = '';
      if (options.namespace && options.namespace !== 'select namespace') {
        qParams += '&namespace=' + options.namespace;
      }

      if (options.dimension && options.dimension !== 'select dimension') {
        qParams += '&dimension=' + options.dimension;
      }

      return this.backendSrv.datasourceRequest({
        url: this.buildSpotinstUrl('/metrics') + qParams,
        data: null,
        method: 'GET',
        headers: this.headers
      }).then(this.mapToTextValue);
    }
  }, {
    key: 'getDimensions',
    value: function getDimensions(options) {
      var qParams = '';
      if (options.namespace && options.namespace !== 'select namespace') {
        qParams += '&namespace=' + options.namespace;
      }

      if (options.metricName && options.metricName !== 'select metric') {
        qParams += '&metric=' + options.metricName;
      }

      return this.backendSrv.datasourceRequest({
        url: this.buildSpotinstUrl('/dimensions') + qParams,
        data: null,
        method: 'GET',
        headers: this.headers
      }).then(this.mapToTextValue);
    }
  }, {
    key: 'mapToTextValue',
    value: function mapToTextValue(result) {
      return _lodash2.default.map(result.data, function (d, i) {
        if (d && d.text && d.value) {
          return {
            text: d.text,
            value: d.value
          };
        } else if (_lodash2.default.isObject(d)) {
          return {
            text: d,
            value: i
          };
        }
        return {
          text: d,
          value: d
        };
      });
    }
  }, {
    key: 'buildSpotinstUrl',
    value: function buildSpotinstUrl(path) {
      return this.url + path + this.accountIdQueryParam;
    }
  }]);

  return GenericDatasource;
}();
//# sourceMappingURL=datasource.js.map
